/*****************************
* Source : regexp.prg           
* System : General
* Author : Phil Ide           
* Created: 09/08/2002             
*                             
* Purpose: Interface to PCRE (Perl-Compatible Regular Expression) Library.
* ----------------------------
* History:                    
* ----------------------------
*    09/08/2002 23:04 PPI - Created
*    30/05/2003 09:58 PPI - Updated with Evgeny's enhancements
*                           (this basically takes advantage of a newer
*                            PCRE.DLL with better memory handling)
*
*****************************/
                              
#include "Common.ch"
#include "dll.ch"
#include "xbpcre.ch"

CLASS RegExp FROM pcre_base
   EXPORTED:
      VAR error

   HIDDEN:
      VAR ePattern
      VAR nOptions
      VAR extra_block
      VAR cPattern
      VAR study
      VAR nSubPatterns
      VAR results

   EXPORTED:
      METHOD init
      METHOD result
      METHOD study
      METHOD exec
      METHOD destroy                               // evg 5/28/2003                                      

ENDCLASS

METHOD RegExp:init( cExp, nOpts )
   local cErr := space(50)
   local nErrOff := 0

   default nOpts to 0

   ::pcre_base:init()

   ::cPattern := cExp
   ::nOptions := nOpts

   ::ePattern := ::pcre_compile(cExp, ::nOptions, @cErr, @nErrOff, NIL)

   if nErrOff > 0
      ::error := {cErr,nErrOff}
   endif
   return (self)

METHOD RegExp:study()
   local cErr := space(50)

   ::study := ::pcre_study(::ePattern,0,@cErr)
   return (self)

METHOD RegExp:exec( cSubj, nOffSet, nOptions )
   local nInt := 10
   local nSize
   local cBin
   local nRet
   local i

   default nOptions to 0,;
           nOffset to 0

   if ::pcre_fullinfo(::ePattern,::study,2,@nInt) == 0
      nInt++
      ::nSubPatterns := nInt
   endif

   cBin := Replicate(Chr(0),nInt*3*4)

   nRet := ::pcre_exec(::ePattern,::study,cSubj,len(cSubj),nOffSet,nOptions,@cBin,nInt*3)

   if nRet > 0
      ::results := Array(nRet*2)
      for i := 1 to nRet*2
         ::results[i] := Bin2L( substr( cBin, (i-1)*4+1, 4 ) )
      next
   endif
   return (nRet)

METHOD RegExp:result(n)
   local aRet[2]
   local nEnd
   if pcount() == 0 .or. n == 0
      aRet := len(::results)
      if aRet > 0
         aRet := aRet/2
      endif
   elseif n <= len(::results)
      aRet[1] := ::results[(n*2)-1]+1
      nEnd    := ::results[n*2]+1
      aRet[2] := nEnd-aRet[1]
   endif
   return (aRet)

METHOD RegExp:destroy()                                      // evg 5/28/2003 
   if ::study <> nil
      ::regfree(@::study)
      ::study      := Nil
   endif
   ::regfree(@::ePattern)
   ::ePattern      := NIL
   ::nOptions      := Nil
   ::extra_block   := Nil
   ::cPattern      := Nil
   ::nSubPatterns  := Nil
   ::results       := Nil
RETURN self
